@@ -138,9 +138,10 @@ Alternatively set key_name/key_secret in the configuration file""")
 
     # Grab the secret key
     secret = parsed_key_file.values()[0]['secret'].strip('\"')
+    algorithm = parsed_key_file.values()[0]['algorithm'].strip('\"')
     f.close()
 
-    return (key_name, secret)
+    return (key_name, algorithm, secret)
 
 
 def query_NS_record(domain_name):
@@ -152,10 +153,13 @@ Return a list of nameserver IPs (might be empty)
     for i in range(0, len(name_list)):
         nameservers = []
         try:
-            for ns in [rdata.target.to_unicode()
-                       for rdata in dns.resolver.query('.'.join(name_list[i:]),
-                                                       'NS')]:
-                nameservers += [_.to_text() for _ in dns.resolver.query(ns)]
+            fqdn = '.'.join(name_list[i:])
+            for rdata in dns.resolver.query(fqdn, dns.rdatatype.NS):
+                ns = rdata.target.to_unicode()
+                nsL = [] 
+                nsL.extend([_.to_text() for _ in dns.resolver.query(ns)]) # default type: A
+                nsL.extend([_.to_text() for _ in dns.resolver.query(ns, rdtype=dns.rdatatype.AAAA)])
+                nameservers.append(nsL)
         except (dns.resolver.NoAnswer, dns.resolver.NXDOMAIN) as e:
             continue
         if nameservers:
@@ -197,7 +201,7 @@ Return True if the record could be verified, false otherwise.
                        rtype,
                        rdata if rdata is not None else "*",
                        ns))
-        resolver.nameservers = [ns]
+        resolver.nameservers = ns
         answer = []
         try:
             answer = [_.to_text().strip('"'+"'")
@@ -502,6 +506,7 @@ def ensure_config_dns(cfg):
 """
     # (str)key_name
     # (str)key_secret
+    # (str)key_algorithm
     # (str)name_server_ip
     # (int)ttl
     # (float)wait
@@ -510,7 +515,7 @@ def ensure_config_dns(cfg):
         key_name = cfg["key_name"]
         key_secret = cfg["key_secret"]
     except KeyError:
-        (key_name, key_secret) = get_isc_key()
+        (key_name, key_algorithm, key_secret) = get_isc_key()
 
     keyringd = {key_name: key_secret}
     keyring = dns.tsigkeyring.from_text(keyringd)
@@ -519,7 +524,7 @@ def ensure_config_dns(cfg):
     try:
         algo = cfg["key_algorithm"]
     except KeyError:
-        algo = ""
+        algo = key_algorithm if key_algorithm else ""
     algo = get_key_algo(algo)
     cfg["keyalgorithm"] = algo
 
